-------------------------------------------------------------------------------
-- 
-------------------------------------------------------------------------------
-- uc2_reset - entity/architecture pair
-------------------------------------------------------------------------------
--
--  ***************************************************************************
--  **  Copyright (c) 2005 by Xilinx, Inc. All rights reserved.              **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        uc2_reset.vhd
-- 
-- Description:     UC2 processor reset module.
--                  
--
-------------------------------------------------------------------------------
-- Structure:   This section should show the hierarchical structure of the 
--              designs. Separate lines with blank lines if necessary to improve
--              readability.
--
--              proc_sys_reset.vhd
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_cmb" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
library UNISIM;
use UNISIM.vcomponents.all;

-------------------------------------------------------------------------------
-- Port Declaration
-------------------------------------------------------------------------------
-- Definition of Generics:
--
--
-- Definition of Ports:
--          UC2_sync_clk                -- Clock 
--          Ext_Reset_In                -- External Reset Input
--          Core_Reset_Req              -- PPC Core reset request                
--          Chip_Reset_Req              -- PPC Chip reset request 
--          System_Reset_Req            -- PPC System reset request
--          Reset405out                 -- PPC core reset out, chip reset out and system reset out
--
-------------------------------------------------------------------------------
entity uc2_reset is

  port (
    UC2_sync_clk     : in  std_logic;
    Ext_Reset_In     : in  std_logic;
    Core_Reset_Req   : in  std_logic;
    Chip_Reset_Req   : in  std_logic;
    System_Reset_Req : in  std_logic;
    Reset405out      : out std_logic
    );

end entity uc2_reset;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture imp of uc2_reset is

-------------------------------------------------------------------------------
-- Constant Declarations
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- Signal and Type Declarations
  signal reset         : std_logic;     -- combinatorial reset in
  signal reset_d1      : std_logic;     -- delayed combinatorial reset in
  signal reset_d2      : std_logic;     -- delayed combinatorial reset in
  signal reset_d16     : std_logic;     -- delayed through SRL16 combinatorial reset in
  signal reset_d17     : std_logic;     -- gated reset from SRL16, as it holds
                                        -- off clearing reset if Ext_Reset_In is very long
  signal sticky_reset  : std_logic;
  signal Reset405out_i : std_logic;

  attribute INIT          : string;
  attribute INIT of SRL_I : label is "0000";
  attribute INIT of FD_d1 : label is "1";

-------------------------------------------------------------------------------
-- Component Declarations
-------------------------------------------------------------------------------

  component FD
-- synthesis translate_off
    generic (
      INIT :     bit := '1');
-- synthesis translate_on
    port (
      Q    : out std_logic;
      C    : in  std_logic;
      D    : in  std_logic);
  end component;

  component FDR
-- synthesis translate_off
    generic (
      INIT :     bit '0');
-- synthesis translate_on
    port (
      Q    : out std_logic;
      C    : in  std_logic;
      D    : in  std_logic;
      R    : in  std_logic);
  end component;


  component SRL16 is
                    -- pragma translate_off
                    generic (
                      INIT :     bit_vector := X"0000"
                      );
                  -- pragma translate_on    
                  port (
                    D      : in  std_logic;
                    Clk    : in  std_logic;
                    A0     : in  std_logic;
                    A1     : in  std_logic;
                    A2     : in  std_logic;
                    A3     : in  std_logic;
                    Q      : out std_logic);
  end component SRL16;



begin

-------------------------------------------------------------------------------
--  Double register to sync up with slowest_sync_clk
-------------------------------------------------------------------------------

  reset <= (Ext_Reset_In or Core_Reset_Req or Chip_Reset_Req or System_Reset_Req);


-- by setting the init value to 1 this will generate the power up reset


  FD_d1    : FD
-- synthesis translate_off
    generic (
      INIT : bit := '1');
-- synthesis translate_on
  port map (
    C => UC2_sync_clk,
    D => reset,
    Q => reset_d1
    );

  FD_d2 : FD
    port map (
      C => UC2_sync_clk,
      D => reset_d1,
      Q => reset_d2
      );


  SRL_I : SRL16
-- synthesis translate_off 
    generic map (
      INIT => X"0000")
-- synthesis translate_on 
    port map (
      Clk  => UC2_sync_clk,
      D    => reset_d2,
      A0   => '1',
      A1   => '1',
      A2   => '1',
      A3   => '1',
      Q    => reset_d16
      );

  sticky_reset <= (Reset405out_i or reset_d2);

  FDRS_I : FDR
    port map(
      C => UC2_sync_clk,
      D => sticky_reset,
      Q => Reset405out_i,
      R => reset_d17
      );


  Reset405out <= Reset405out_i;
  reset_d17   <= (reset_d16 and not reset_d2);

end imp;
